svg.learning-curve {
    width: 100%;
    shape-rendering: crispEdges;
    font-size: 14px;
}

svg.learning-curve .tick text {
    font-size: 14px;
    fill: #505050;
}

svg.learning-curve .tick line {
    stroke-width: 2;
    stroke: #505050;
}

svg.learning-curve .background {
    fill: #EBEBEB;
}

svg.learning-curve .facet {
    font-size: 14px;
}

svg.learning-curve .facet-background {
    fill: #D9D9D9;
}

svg.learning-curve path.line {
    fill: none;
    shape-rendering: geometricPrecision;
    stroke-width: 1;
}

svg.learning-curve .grid line {
    stroke: #fff;
    stroke-width: 2;
}

svg.learning-curve .grid .minor {
    stroke-opacity: .5;
}

svg.learning-curve .grid text {
    display: none;
}

svg.learning-curve .axis path,
svg.learning-curve .grid path {
    display: none;
}

svg.learning-curve .axis.hide-axis {
    display: none;
}

svg.learning-curve .legned rect {
    fill: #EBEBEB;
}

svg.learning-curve .legned line {
    stroke-width: 2;
}
